"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const createRule_1 = __importDefault(require("../util/createRule"));
const QUERY_HOOK_PATTERN = /^use(.*)Query$/g;
const handleQueryError = (0, createRule_1.default)({
    name: 'handle-query-error',
    meta: {
        docs: {
            description: 'Error handling is important :)',
            recommended: 'warn',
        },
        messages: {
            general: 'Handle error returned from this query',
            lazyQuery: 'Handle error returned from this lazy query',
        },
        type: 'problem',
        schema: [
            {
                type: 'object',
                properties: {
                    ignoreHooks: {
                        type: 'array',
                    },
                },
                additionalProperties: false,
            },
        ],
    },
    defaultOptions: [{ ignoreHooks: [] }],
    create(context) {
        return {
            VariableDeclarator(node) {
                var _a, _b;
                if (((_a = node.init) === null || _a === void 0 ? void 0 : _a.type) !== utils_1.AST_NODE_TYPES.CallExpression) {
                    return;
                }
                if (node.init.callee.type !== utils_1.AST_NODE_TYPES.Identifier) {
                    return;
                }
                if (!node.init.callee.name.match(QUERY_HOOK_PATTERN)) {
                    return;
                }
                const ignoredHooks = (_b = context.options[0]) === null || _b === void 0 ? void 0 : _b.ignoreHooks;
                if (ignoredHooks &&
                    ignoredHooks.includes(node.init.callee.name)) {
                    return;
                }
                if (node.id.type === utils_1.AST_NODE_TYPES.ObjectPattern) {
                    if (!node.id.properties.some(property => {
                        if (property.type !== utils_1.AST_NODE_TYPES.Property ||
                            property.key.type !== utils_1.AST_NODE_TYPES.Identifier) {
                            return false;
                        }
                        return (property.key.name === 'error' ||
                            property.key.name === 'errors');
                    })) {
                        context.report({
                            node,
                            messageId: 'general',
                        });
                    }
                }
                else if (node.id.type === utils_1.AST_NODE_TYPES.ArrayPattern) {
                    const objectElement = node.id.elements[1];
                    if (!objectElement ||
                        objectElement.type !== utils_1.AST_NODE_TYPES.ObjectPattern) {
                        context.report({
                            node,
                            messageId: 'lazyQuery',
                        });
                        return;
                    }
                    if (!objectElement.properties.some(property => {
                        if (property.type !== utils_1.AST_NODE_TYPES.Property ||
                            property.key.type !== utils_1.AST_NODE_TYPES.Identifier) {
                            return false;
                        }
                        return (property.key.name === 'error' ||
                            property.key.name === 'errors');
                    })) {
                        context.report({
                            node,
                            messageId: 'lazyQuery',
                        });
                    }
                }
            },
        };
    },
});
exports.default = handleQueryError;
